import pandas as pd
from pylab import *
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import argparse
from math import *
import numpy as np
from matplotlib.offsetbox import TextArea, VPacker, AnnotationBbox
import matplotlib.patches as patches


def clothing_insulation(icl):
    """ Change unit of measure of clothing insulation: "clo" to "m*m*K/W"
            Parameter:
                - icl(float): clothing insulation  expressed in "clo"
            Return:
                - Icl(float): clothing insulation  expressed in "m*m*K/W"
    """
    Icl = icl*0.155
    return Icl


def clothing_surface_factor(Icl):
    """ Calculates the clothing surface area factor according to
            the values of clothing insulation
            Parameter:
                - Icl(float): clothing insulation
            Return:
                - fcl(float): clothing surface area factor
    """
    if Icl > 0.078:
        fcl = 1.05+0.645*Icl
    else:
        fcl = 1.00+1.290*Icl
    return fcl


def convective_heat(var):
    """ Calculates the convective heat transfer coefficient
            Parameters:
                - var(float): relative air velocity
            Return :
                - hc(float): convective heat transfer coefficient
    """
    hc = 12.1*np.sqrt(var)
    return hc


def clothing_surface_temperature(M, Icl, ta, fcl, hc, tr, W):
    """Calculates the  clothing surface temperature
            Parameters:
                - M(float): metabolic rate
                - Icl(float): clothing insulation
                - ta(float):  air temperature
                - fcl(float): clothing surface factor
                - hc(float): convective heat transfer coefficient
                - tr(float): mean radiant temperature
                - W(float): effective mechanical power
            Return :
                - tcl(float): clothing surface temperature
    """
    MW = M-W
    Y = -Icl*3.96*(10**(-8)*fcl)
    a4 = 35.7-0.028*MW+Icl*(3.96*(10**(-8)*fcl*(tr+273)**4))+Y*273**4+Icl*fcl*hc*ta
    a0 = -Y
    a1 = -Y*4*273
    a2 = -Y*6*273**2
    a3 = (1+Icl*fcl*hc-Y*4*273**3)
    coeff = [a0, a1, a2, a3, -a4]
    co = np.roots(coeff)
    for i in co:
        if i > 0 and i < 50:
            tcl = i
    return tcl


def water_vapor_pressure(ta, hum):
    """ Calculates the  water vapor partial pressure
            Parameters:
                - ta(float): air temperature
                - hum(float): humidity
            Return :
                - pa(float): water vapor partial pressure
    """
    if ta < 0:
        psat = 611*np.exp((22.44*ta)/(272.44+ta))
    else:
        psat = 611*np.exp((17.08*ta)/(234.18+ta))
    pa = (psat*hum)/100
    return pa


def PMV(M, pa, tcl, ta, fcl, hc, tr, W):
    """ Calculates the  PMV
            Parameters:
                - M(float): metabolic rate
                - pa(list): water vapor partial pressure
                - tcl(list): clothing surface temperature
                - ta(list): air temperature
                - fcl(list): clothing surface factor
                - hc(float): convective heat transfer coefficient
                - tr(list): mean radiant temperature
                - W(float): effective mechanical power
            Return :
                - pmv(list): PMV
    """
    MW = M-W
    x1 = 0.303*exp(-0.036*M)+0.028
    x2 = 5733-6.995*(MW)-pa
    x3 = -1.7*10**(-5)*M*(5867-pa)-0.0014*M*(34-ta)
    x4 = (tcl+273)**4-(tr+273)**4
    x5 = fcl*hc*(tcl-ta)
    y = x1*(MW-3.05*10**(-3)*x2-0.42*(MW-58.15)+x3-3.96*10**(-8)*fcl*x4-x5)
    return y


def PPD(PMV):
    """ Calculates the ppd indice
            Parameter :
                - PMV(list): pmv indice
            Return :
                - ppd(list): ppd indice
    """
    pdd = 100-95*np.exp(-0.03353*PMV**4-0.2179*PMV**2)
    return pdd


def verify_pmv(real_pmv, calculated_pmv):
    """ Verify if real_pmv equal to the calculated_pmv.
            Parameters :
                - real_pmv(float): real value of pmv
                - calculated_pmv(float): calculated value of pmv
            Return :
                - verif(str): equal to "correct value" if the two values
                            correspond else equal to "incorrect value"
    """
    error = np.abs(real_pmv-calculated_pmv)
    verif = ""
    if error < 0.011:
        verif = "correct value"
    else:
        verif = "incorrect value"
    return verif


if '__main__':
    W = 0
    ta = [22, 27, 27, 23.5, 23.5, 19, 23.5, 23.5, 23, 23, 22, 27, 27]
    tr = [22, 27, 27, 25.5, 25.5, 19, 23.5, 23.5, 21, 21, 22, 27, 27]
    var = [0.1, 0.1, 0.3, 0.1, 0.3, 0.1, 0.1, 0.3, 0.1, 0.3, 0.1, 0.1, 0.3]
    hum = [60, 60, 60, 60, 60, 40, 40, 40, 40, 40, 60, 60, 60]
    real_pmv = [-0.75, 0.77, 0.44, -0.01, -0.55, -0.60, 0.5, 0.12, 0.05,
                -0.16, 0.05, 1.17, 0.95]
    M = [1.2*58, 1.2*58, 1.2*58, 1.2*58, 1.2*58, 1.2*58, 1.2*58, 1.2*58,
         1.2*58, 1.2*58, 1.6*58, 1.6*58, 1.6*58]
    icl = [0.5, 0.5, 0.5, 0.5, 0.5, 1, 1, 1, 1, 1, 0.5, 0.5, 0.5]
    print("ta(°C) | tr(°C) | var(m/s)  | RH(%)  | M(W/m^2)  |"
          r"$Icl(m^2K/W)$" "| pmv | ppd(%) | verify pmv")
    for i in range(len(ta)):
        Icl = clothing_insulation(icl[i])
        fcl = clothing_surface_factor(Icl)
        hc = convective_heat(var[i])
        tcl = clothing_surface_temperature(M[i], Icl, ta[i], fcl, hc, tr[i], W)
        pa = water_vapor_pressure(ta[i], hum[i])
        pmv = PMV(M[i], pa, tcl, ta[i], fcl, hc, tr[i], W)
        pmv = float(np.round(np.real(pmv), 2))
        ppd = PPD(pmv)
        ppd = round(ppd, 2)
        verify = verify_pmv(real_pmv[i], pmv)
        print('case '+str(i), '|', ta[i], '|', tr[i], ' |', var[i], '|',
              hum[i], '|', M[i], '|', Icl, '|', pmv, '|', ppd, '|', verify)
